#include "game.h"
#include "exit.h"
#include "map.h"
#include "player.h"
#include "cfg.h"
#include "spike.h"
#include <string.h>

void
game_init(Game *this)
{
	memset(this, 0, sizeof(*this));
	game_restart_scene(this);
}

void
game_deinit(Game *this)
{
	(void)this;
}

void
game_update(Game *this)
{
	if (this->queue_restart_scene > 0) {
		if (--this->queue_restart_scene == 0)
			game_restart_scene(this);
	}
	if (this->queue_next_scene) {
		this->queue_next_scene = false;
		map_next();
		game_restart_scene(this);
		return;
	}
	for (int i = 0; i < MAX_ENTITIES; i++) {
		Entity *const e = &this->entities[i];
		if (e->type != ET_NONE && e->update != NULL)
			e->update(e, this);
	}
}

void
game_draw(Game *this)
{
	map_draw();
	for (int i = 0; i < MAX_ENTITIES; i++) {
		Entity *const e = &this->entities[i];
		if (e->type != ET_NONE && e->draw != NULL)
			e->draw(e, this);
	}
}

void
game_restart_scene(Game *this)
{
	memset(this->entities, 0, sizeof(this->entities));
	for (int y = 0; y < map_height(); y++)
		for (int x = 0; x < map_width(); x++) {
			const int dx = x * TSIZE + TSIZE / 2;
			const int dy = y * TSIZE + TSIZE / 2;
			switch (map_get(x, y)) {
			case 2:
				player_init(game_create_entity(this), dx, dy);
				break;
			case 4:
				exit_init(game_create_entity(this), dx, dy);
				break;
			case 5:
				spike_init(game_create_entity(this), dx, dy);
				break;
			default:
				break;
			}
		}
}

Entity *
game_create_entity(Game *this)
{
	Entity *e = &this->entities[MAX_ENTITIES - 1];
	for (int i = 0; i < MAX_ENTITIES; i++)
		if (this->entities[i].type == ET_NONE) {
			e = &this->entities[i];
			break;
		}
	e->type = ET_NONE;
	e->uuid = this->uuid;
	this->uuid += 1;
	return e;
}

int
game_entity_count(Game *this, unsigned int type)
{
	int count = 0;
	for (int i = 0; i < MAX_ENTITIES; i++)
		count += (this->entities[i].type == type);
	return count;
}
